@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2021 NXP
+# Copyright 2020-2022 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """Contains support for BEE encryption for RT10xx devices."""
@@ -53,12 +53,15 @@ class BeeBaseClass:
         return self.__class__._size()
 
     def info(self) -> str:
-        """:return: text description of the instance."""
-        raise NotImplementedError("abstract method")
+        """Info method.
+
+        :return: text description of the instance.
+        :raises NotImplementedError: Derived class has to implement this method
+        """
+        raise NotImplementedError("Derived class has to implement this method.")
 
     def update(self) -> None:
         """Updates internal fields of the instance."""
-        pass
 
     def validate(self) -> None:
         """Validates the configuration of the instance.
@@ -207,7 +210,7 @@ class BeeProtectRegionBlock(BeeBaseClass):
         )
 
         # - FAC regions, 1 - 4
-        self.fac_regions: List[BeeFacRegion] = list()
+        self.fac_regions: List[BeeFacRegion] = []
 
     def update(self) -> None:
         """Updates start and end address of the encryption region."""
@@ -252,9 +255,9 @@ class BeeProtectRegionBlock(BeeBaseClass):
         if self._start_addr > self._end_addr or self._end_addr > 0xFFFFFFFF:
             raise SPSDKError("Invalid start/end address")
         if self.mode != BeeProtectRegionBlockAesMode.CTR:
-            raise SPSDKError("Only AES/CTR encryption mode supported now")  # TODO
+            raise SPSDKError("Only AES/CTR encryption mode supported now")
         if len(self.counter) != 16:
-            raise SPSDKError("Invalid conter")
+            raise SPSDKError("Invalid counter")
         if self.counter[-4:] != b"\x00\x00\x00\x00":
             raise SPSDKError("last four bytes must be zero")
         if self.fac_count <= 0 or self.fac_count > self.FAC_REGIONS:
@@ -472,7 +475,7 @@ class BeeRegionHeader(BeeBaseClass):
 
         The result is ordered, first value should be burned to the lowest address.
         """
-        result = list()
+        result = []
         for pos in range(16, 0, -4):
             result.append(unpack_from(">I", self._sw_key[pos - 4 : pos])[0])
         return result
