@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2017-2018 Martin Olejar
-# Copyright 2019-2021 NXP
+# Copyright 2019-2022 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -97,8 +97,11 @@ class BootImgBase:
         self._dcd = value
 
     def info(self) -> str:
-        """Text info about the instance."""
-        raise NotImplementedError()
+        """Text info about the instance.
+
+        :raises NotImplementedError: Derived class has to implement this method
+        """
+        raise NotImplementedError("Derived class has to implement this method.")
 
     def add_image(self, data: bytes, img_type: EnumAppType, address: int) -> None:
         """Add specific image into the main boot image.
@@ -106,12 +109,16 @@ class BootImgBase:
         :param data: Raw binary data of the application image
         :param img_type: see EnumAppType
         :param address: TBD
+        :raises NotImplementedError: Derived class has to implement this method
         """
-        raise NotImplementedError()
+        raise NotImplementedError("Derived class has to implement this method.")
 
     def export(self) -> bytes:
-        """Binary representation of the instance (serialization)."""
-        raise NotImplementedError()
+        """Binary representation of the instance (serialization).
+
+        :raises NotImplementedError: Derived class has to implement this method
+        """
+        raise NotImplementedError("Derived class has to implement this method.")
 
     @classmethod
     def parse(
@@ -120,8 +127,11 @@ class BootImgBase:
         step: int = 0x100,
         size: Optional[int] = None,
     ) -> "BootImgBase":
-        """Parse of IMX Boot Image Base."""
-        raise NotImplementedError()
+        """Parse of IMX Boot Image Base.
+
+        :raises NotImplementedError: Derived class has to implement this method
+        """
+        raise NotImplementedError("Derived class has to implement this method.")
 
 
 ########################################################################################################################
@@ -158,7 +168,7 @@ class BootImgRT(BootImgBase):
     # The length of BDT segment
     BDT_SIZE = 0x20
     # The length of DEK key section; Note: Dek key is just 16 bytes
-    DEK_SIZE = 0x200  # TODO this is sector size alignment???
+    DEK_SIZE = 0x200  # Is this sector size alignment???
 
     def __init__(
         self,
@@ -513,9 +523,9 @@ class BootImgRT(BootImgBase):
         entry_addr = unpack_from("<I", data, 4)[0]
         if entry_addr == 0:  # there can be padding for images located in RAM, see flashloader
             entry_addr = address
-            if not entry_addr > 0:
+            if entry_addr <= 0:
                 raise SPSDKError("entry_addr not detected from image, must be specified explicitly")
-        elif (address >= 0) and (address != entry_addr):
+        elif address >= 0 and address != entry_addr:
             raise SPSDKError("entry_address does not match with the image")
         self._ivt.app_address = entry_addr
         self.app.data = data
@@ -983,7 +993,7 @@ class BootImgRT(BootImgBase):
             stream = BytesIO(stream)
 
         if not isinstance(stream, (BufferedReader, BytesIO)):
-            raise SPSDKError(' Not correct value type: "{}" !'.format(type(stream)))
+            raise SPSDKError(f' Not correct value type: "{type(stream)}" !')
 
         header, start_pos, end_pos = cls._find_ivt_pos(stream, size)
 
@@ -1591,7 +1601,7 @@ class BootImg8m(BootImgBase):
         obj.app.data = read_raw_data(stream, app_size, app_start)
         obj.app.padding = 0
         # Parse CSF
-        # TODO finalize the code below
+        # Finalize the code below
         # if obj.ivt.csf_address:
         #    obj.csf = SegCSF.parse(buffer)
         #    obj.csf.padding = obj.bdt.length - ((obj.ivt.csf_address - obj.ivt.ivt_address) + obj.csf.size)
@@ -1779,7 +1789,7 @@ class BootImg3a(BootImgBase):
         msg += "#" * 60 + "\n\n"
         for index, ivt in enumerate(self.ivt):
             msg += "-" * 60 + "\n"
-            msg += "- IVT[{}]\n".format(index)
+            msg += f"- IVT[{index}]\n"
             msg += "-" * 60 + "\n\n"
             msg += ivt.info()
         # Print BDI
@@ -1788,7 +1798,7 @@ class BootImg3a(BootImgBase):
         msg += "#" * 60 + "\n\n"
         for index, bdi in enumerate(self.bdt):
             msg += "-" * 60 + "\n"
-            msg += "- BDI[{}]\n".format(index)
+            msg += f"- BDI[{index}]\n"
             msg += "-" * 60 + "\n\n"
             msg += bdi.info()
         # Print DCD
@@ -1947,7 +1957,6 @@ class BootImg3a(BootImgBase):
             raise SPSDKError("Not an i.MX Boot Image!")
 
         obj = cls(version=header.param)
-        # TODO: not used right now: img_size = last_index - start_index
         if start_index > 0:
             obj.offset = start_index
         # Parse IVT
@@ -2182,7 +2191,7 @@ class BootImg3b(BootImgBase):
         msg += "#" * 60 + "\n\n"
         for index, ivt in enumerate(self.ivt):
             msg += "-" * 60 + "\n"
-            msg += "- IVT[{}]\n".format(index)
+            msg += f"- IVT[{index}]\n"
             msg += "-" * 60 + "\n\n"
             msg += ivt.info()
         # Print BDI
@@ -2191,7 +2200,7 @@ class BootImg3b(BootImgBase):
         msg += "#" * 60 + "\n\n"
         for index, bdi in enumerate(self.bdt):
             msg += "-" * 60 + "\n"
-            msg += "- BDI[{}]\n".format(index)
+            msg += f"- BDI[{index}]\n"
             msg += "-" * 60 + "\n\n"
             msg += bdi.info()
         # Print DCD
@@ -2357,7 +2366,6 @@ class BootImg3b(BootImgBase):
             raise SPSDKError("Not an i.MX Boot Image!")
 
         obj = cls(version=header.param)
-        # TODO: not used right now: img_size = last_index - start_index
         if start_index > 0:
             obj.offset = start_index
         # Parse IVT
@@ -2432,8 +2440,11 @@ class BootImg4(BootImgBase):
         return msg
 
     def add_image(self, data: bytes, img_type: int, address: int) -> None:
-        """Not implemented."""
-        raise NotImplementedError()
+        """Add image.
+
+        :raises NotImplementedError: Not yet implemented
+        """
+        raise NotImplementedError("Not yet implemented.")
 
     def export(self) -> bytes:
         """Export."""
@@ -2441,7 +2452,7 @@ class BootImg4(BootImgBase):
         data = bytes()
         data += self._cont1_header.export()
         data += self._cont2_header.export()
-        # TODO: Complete Implementation
+        # Complete Implementation
         return data
 
     @classmethod
@@ -2464,7 +2475,7 @@ class BootImg4(BootImgBase):
             stream = BytesIO(stream)
 
         if not isinstance(stream, (BufferedReader, BytesIO)):
-            raise SPSDKError(' Not correct value type: "{}" !'.format(type(stream)))
+            raise SPSDKError(f" Not correct value type: '{type(stream)}' !")
 
         start_index = stream.tell()
         last_index = stream.seek(0, SEEK_END)
@@ -2485,7 +2496,6 @@ class BootImg4(BootImgBase):
         if not imx_image:
             raise SPSDKError(" Not an i.MX Boot Image !")
 
-        # TODO: not used right now: img_size = last_index - start_index
         obj = cls()
         if start_index > 0:
             obj.offset = start_index
@@ -2493,7 +2503,7 @@ class BootImg4(BootImgBase):
         # Parse Containers
         obj._cont1_header = SegBIC1.parse(read_raw_data(stream, 0x400))
         obj._cont2_header = SegBIC1.parse(read_raw_data(stream, 0x400))
-        # TODO: Complete Implementation
+        # Complete Implementation
         return obj
 
 
@@ -2569,7 +2579,6 @@ class KernelImg:
 
     def info(self) -> None:
         """String representation of the IMX Kernel Image."""
-        pass
 
     def export(self) -> bytes:
         """Export."""
@@ -2610,7 +2619,7 @@ def parse(
         stream = BytesIO(stream)
 
     if not isinstance(stream, (BufferedReader, BytesIO)):
-        raise SPSDKError('Not correct value type: "{}" !'.format(type(stream)))
+        raise SPSDKError(f"Not correct value type: '{type(stream)}' !")
 
     # calculate stream size
     start_index = stream.tell()
