@@ -1,15 +1,14 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2021 NXP
+# Copyright 2020-2022 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """Module to handle registers descriptions with support for XML files."""
 
 import logging
 import xml.etree.ElementTree as ET
-from math import ceil
-from typing import Any, Callable, Dict, List, Mapping, Optional, Union
+from typing import Any, Callable, Dict, List, Mapping, Union
 from xml.dom import minidom
 
 from jinja2 import Environment, FileSystemLoader
@@ -23,7 +22,8 @@ from spsdk.utils.exceptions import (
     SPSDKRegsErrorRegisterGroupMishmash,
     SPSDKRegsErrorRegisterNotFound,
 )
-from spsdk.utils.misc import format_value, value_to_bool, value_to_bytes, value_to_int
+from spsdk.utils.images import BinaryImage, BinaryPattern
+from spsdk.utils.misc import format_value, value_to_bool, value_to_bytes, value_to_int, write_file
 
 HTMLDataElement = Mapping[str, Union[str, dict, list]]
 HTMLData = List[HTMLDataElement]
@@ -41,12 +41,13 @@ class RegsEnum:
         :param value: Value of enumeration.
         :param description: Text description of enumeration.
         :param max_width: Maximal width of enum value used to format output
+        :raises SPSDKRegsError: Invalid input value.
         """
         self.name = name or "N/A"
         try:
-            self.value = value_to_bytes(value)
-        except (TypeError, ValueError, SPSDKError):
-            self.value = b""
+            self.value = value_to_int(value)
+        except (TypeError, ValueError, SPSDKError) as exc:
+            raise SPSDKRegsError(f"Invalid Enum Value: {value}") from exc
         self.description = description or "N/A"
         self.max_width = max_width
 
@@ -55,7 +56,7 @@ class RegsEnum:
         """Initialization Enum by XML ET element.
 
         :param xml_element: Input XML subelement with enumeration data.
-        :param maxwidth: The maximal width of bitfield for this enum (used for formating).
+        :param maxwidth: The maximal width of bitfield for this enum (used for formatting).
         :return: The instance of this class.
         :raises SPSDKRegsError: Error during enum XML parsing.
         """
@@ -65,7 +66,7 @@ class RegsEnum:
 
         raw_val = xml_element.attrib["value"]
         try:
-            value = value_to_bytes(raw_val)
+            value = value_to_int(raw_val)
         except (TypeError, ValueError, SPSDKError) as exc:
             raise SPSDKRegsError(f"Invalid Enum Value: {raw_val}") from exc
 
@@ -78,18 +79,14 @@ class RegsEnum:
 
         :return: Integer value of Enum.
         """
-        return int.from_bytes(self.value, "big")
+        return self.value
 
     def get_value_str(self) -> str:
-        """Method returns formated value.
+        """Method returns formatted value.
 
         :return: Formatted string with enum value.
         """
-        if self.value == b"":
-            return "N/A"
-
-        val = self.get_value_int()
-        return format_value(val, self.max_width)
+        return format_value(self.value, self.max_width)
 
     def add_et_subelement(self, parent: ET.Element) -> None:
         """Creates the register XML structure in ElementTree.
@@ -126,6 +123,7 @@ class RegsBitField:
         description: str = None,
         reset_val: Any = "0",
         access: str = "RW",
+        hidden: bool = False,
     ) -> None:
         """Constructor of RegsBitField class. Used to store bitfield information.
 
@@ -136,6 +134,7 @@ class RegsBitField:
         :param description: Text description of bitfield.
         :param reset_val: Reset value of bitfield.
         :param access: Access type of bitfield.
+        :param hidden: The bitfield will be hidden from standard searches.
         """
         self.parent = parent
         self.name = name or "N/A"
@@ -144,10 +143,8 @@ class RegsBitField:
         self.description = description or "N/A"
         self.reset_value = value_to_int(reset_val, 0)
         self.access = access
+        self.hidden = hidden
         self._enums: List[RegsEnum] = []
-        self.bc_enum: Optional[
-            Callable[["RegsRegister", "RegsBitField", RegsEnum], List[str]]
-        ] = None
         self._update_reset_value()
         self.set_value(self.reset_value, raw=True)
 
@@ -169,10 +166,11 @@ class RegsBitField:
             if "reset_value" in xml_element.attrib
             else 0
         )
-        bitfield = cls(parent, name, offset, width, descr, reset_value, access)
+        hidden = xml_element.tag != "bit_field"
+        bitfield = cls(parent, name, offset, width, descr, reset_value, access, hidden)
 
         if xml_element.text:
-            xml_enums = xml_element.findall(f"bit_field_value")
+            xml_enums = xml_element.findall("bit_field_value")
             for xml_enum in xml_enums:
                 bitfield.add_enum(RegsEnum.from_xml_element(xml_enum, width))
         return bitfield
@@ -194,7 +192,7 @@ class RegsBitField:
     def add_enum(self, enum: RegsEnum) -> None:
         """Add bitfield enum.
 
-        :param enum: New enumeration value for bitfiled.
+        :param enum: New enumeration value for bitfield.
         """
         self._enums.append(enum)
 
@@ -203,7 +201,7 @@ class RegsBitField:
 
         :return: Current value of bitfield.
         """
-        reg_val = int.from_bytes(self.parent.get_value(), "big")
+        reg_val = self.parent.get_value()
         value = reg_val >> self.offset
         mask = (1 << self.width) - 1
         value = value & mask
@@ -226,23 +224,21 @@ class RegsBitField:
         new_val_int = value_to_int(new_val)
         if new_val_int > 1 << self.width:
             raise SPSDKError("The input value is out of bitfield range")
-        reg_val = int.from_bytes(self.parent.get_value(), "big")
+        reg_val = self.parent.get_value()
         mask = ((1 << self.width) - 1) << self.offset
         reg_val = reg_val & ~mask
         value = (new_val_int << self.offset) & mask
         reg_val = reg_val | value
-        reg_val_bytes = reg_val.to_bytes(length=ceil(self.parent.width / 8), byteorder="big")
-        self.parent.set_value(reg_val_bytes, raw)
+        self.parent.set_value(reg_val, raw)
 
     def _update_reset_value(self) -> None:
         """Updates the reset value of the bitfield in register."""
-        reg_val = int.from_bytes(self.parent.get_value(), "big")
+        reg_val = self.parent.get_value()
         mask = ((1 << self.width) - 1) << self.offset
         reg_val = reg_val & ~mask
         value = (self.reset_value << self.offset) & mask
         reg_val = reg_val | value
-        reg_val_bytes = reg_val.to_bytes(length=ceil(self.parent.width / 8), byteorder="big")
-        self.parent._reset_value = reg_val_bytes  # pylint: disable=protected-access
+        self.parent._reset_value = reg_val  # pylint: disable=protected-access
 
     def set_enum_value(self, new_val: str, raw: bool = False) -> None:
         """Updates the value of the bitfield by its enum value.
@@ -252,18 +248,13 @@ class RegsBitField:
         :raises SPSDKRegsErrorEnumNotFound: Input value cannot be decoded.
         """
         try:
-            if new_val not in self.get_enum_names():
-                found_name = self.get_bc_enum(new_val)
-                if found_name:
-                    logger.warning(f"The enum {new_val} has been replaced by {found_name}.")
-                    new_val = found_name
             val_int = self.get_enum_constant(new_val)
         except SPSDKRegsErrorEnumNotFound as exc:
             # Try to decode standard input
             try:
                 val_int = value_to_int(new_val)
             except TypeError:
-                raise exc
+                raise exc  # pylint: disable=raise-missing-from
         self.set_value(val_int, raw)
 
     def get_enum_value(self) -> Union[str, int]:
@@ -302,7 +293,7 @@ class RegsBitField:
 
         :param parent: The parent object of ElementTree.
         """
-        element = ET.SubElement(parent, "bit_field")
+        element = ET.SubElement(parent, "reserved_bit_field" if self.hidden else "bit_field")
         element.set("offset", hex(self.offset))
         element.set("width", str(self.width))
         element.set("name", self.name)
@@ -329,32 +320,6 @@ class RegsBitField:
             "bit_values": enum_desc,
         }
 
-    def enable_backward_compatibility(
-        self,
-        bc_enum: Callable[["RegsRegister", "RegsBitField", RegsEnum], List[str]],
-    ) -> None:
-        """Enables support for basic backward compatibility with older name convention.
-
-        :param bc_enum: Function to provide list of back compatibility names of enum, defaults to None
-        """
-        self.bc_enum = bc_enum
-
-    def get_bc_enum(self, old_name: str) -> Optional[str]:
-        """Try to translate old name of enum with the current one.
-
-        :param old_name: Old name to try to find
-        :return: Name of current enum if exists
-        """
-        if not self.bc_enum:
-            return None
-
-        enum_list = self.get_enums()
-        for enum in enum_list:
-            bc_list = self.bc_enum(self.parent, self, enum)
-            if old_name in bc_list:
-                return enum.name
-        return None
-
     def __str__(self) -> str:
         """Override 'ToString()' to print register.
 
@@ -367,6 +332,8 @@ class RegsBitField:
         output += f"Access:   {self.access} bits\n"
         output += f"Reset val:{self.reset_value}\n"
         output += f"Description: \n {self.description}\n"
+        if self.hidden:
+            output += "This is hidden bitfield!\n"
 
         i = 0
         for enum in self._enums:
@@ -387,6 +354,9 @@ class RegsRegister:
         description: str = None,
         reverse: bool = False,
         access: str = None,
+        config_as_hexstring: bool = False,
+        otp_index: int = None,
+        reverse_subregs_order: bool = False,
     ) -> None:
         """Constructor of RegsRegister class. Used to store register information.
 
@@ -396,6 +366,9 @@ class RegsRegister:
         :param description: Text description of register.
         :param reverse: Multi  register value is stored in reverse order.
         :param access: Access type of register.
+        :param config_as_hexstring: Config is stored as a hex string.
+        :param otp_index: Index of OTP fuse.
+        :param reverse_subregs_order: Reverse order of sub registers.
         """
         self.name = name
         self.offset = offset
@@ -405,12 +378,14 @@ class RegsRegister:
         self.reverse = reverse
         self._bitfields: List[RegsBitField] = []
         self._set_value_hooks: list = []
-        self._value = bytes(width // 8)
-        self._reset_value = bytes(width // 8)
-        self.bc_bitfield: Optional[Callable[["RegsRegister", RegsBitField], List[str]]] = None
+        self._value = 0
+        self._reset_value = 0
+        self.config_as_hexstring = config_as_hexstring
+        self.otp_index = otp_index
+        self.reverse_subregs_order = reverse_subregs_order
 
         # Grouped register members
-        self.sub_regs: List[RegsRegister] = []
+        self.sub_regs: List["RegsRegister"] = []
         self._sub_regs_width_init = False
         self._sub_regs_width = 0
 
@@ -425,14 +400,33 @@ class RegsRegister:
         offset = value_to_int(xml_element.attrib["offset"]) if "offset" in xml_element.attrib else 0
         width = value_to_int(xml_element.attrib["width"]) if "width" in xml_element.attrib else 0
         descr = xml_element.attrib["description"] if "description" in xml_element.attrib else "N/A"
+        reverse_subregs_order = (
+            xml_element.attrib["reverse_subregs_order"]
+            if "reverse_subregs_order" in xml_element.attrib
+            else "False"
+        ) == "True"
         reverse = (
             xml_element.attrib["reversed"] if "reversed" in xml_element.attrib else "False"
         ) == "True"
         access = xml_element.attrib["access"] if "access" in xml_element.attrib else "N/A"
+        otp_index_raw = xml_element.attrib.get("otp_index")
+        otp_index = None
+        if otp_index_raw:
+            otp_index = value_to_int(otp_index_raw)
+        reg = cls(
+            name,
+            offset,
+            width,
+            descr,
+            reverse,
+            access,
+            otp_index=otp_index,
+            reverse_subregs_order=reverse_subregs_order,
+        )
 
-        reg = cls(name, offset, width, descr, reverse, access)
         if xml_element.text:
-            xml_bitfields = xml_element.findall(f"bit_field")
+            xml_bitfields = xml_element.findall("bit_field")
+            xml_bitfields.extend(xml_element.findall("reserved_bit_field"))
             xml_bitfields_len = len(xml_bitfields)
             for xml_bitfield in xml_bitfields:
                 bitfield = RegsBitField.from_xml_element(xml_bitfield, reg)
@@ -515,6 +509,8 @@ class RegsRegister:
         element.set("name", self.name)
         element.set("reversed", str(self.reverse))
         element.set("description", self.description)
+        if self.otp_index:
+            element.set("otp_index", str(self.otp_index))
         for bitfield in self._bitfields:
             bitfield.add_et_subelement(element)
 
@@ -543,8 +539,7 @@ class RegsRegister:
         :raises SPSDKError: When invalid values is loaded into register
         """
         try:
-            value = value_to_bytes(val)
-            value = value.rjust(self.width // 8, b"\x00")
+            value = value_to_int(val)
             if not raw and len(self._set_value_hooks) > 0:
                 for hook in self._set_value_hooks:
                     value = hook[0](value, hook[1])
@@ -553,51 +548,68 @@ class RegsRegister:
 
             if self.has_group_registers():
                 # Update also values in sub registers
-                for index, sub_reg in enumerate(self.sub_regs):
-                    sub_reg.set_value(value[index * 4 : index * 4 + 4])
+                subreg_width = self.sub_regs[0].width
+                for index, sub_reg in enumerate(self.sub_regs, start=1):
+                    if self.reverse_subregs_order:
+                        bit_pos = (index - 1) * subreg_width
+                    else:
+                        bit_pos = self.width - index * subreg_width
+                    sub_reg.set_value((value >> bit_pos) & ((1 << subreg_width) - 1))
 
-        except SPSDKError:
-            logger.error(f"Loaded invalid value {str(val)}")
-            raise SPSDKError(f"Loaded invalid value {str(val)}")
+        except SPSDKError as exc:
+            raise SPSDKError(f"Loaded invalid value {str(val)}") from exc
 
     def reset_value(self, raw: bool = False) -> None:
         """Reset the value of register.
 
         :param raw: Do not use any modification hooks.
         """
-        value = bytearray(int(self.width // 8))
-        int_val = 0
+        value = 0
         for bitfield in self.get_bitfields():
             width = bitfield.width
             offset = bitfield.offset
             val = bitfield.reset_value
-            int_val |= (val & ((1 << width) - 1)) << offset
+            value |= (val & ((1 << width) - 1)) << offset
 
-        self.set_value(int_val.to_bytes(len(value), "big"), raw)
+        self.set_value(value, raw)
 
-    def get_value(self) -> bytes:
+    def get_value(self) -> int:
         """Get the value of register."""
         if self.has_group_registers():
             # Update local value, by the sub register values
-            sub_regs_value = bytearray(self.width // 8)
-            for index, sub_reg in enumerate(self.sub_regs):
-                sub_regs_value[index * 4 : index * 4 + 4] = sub_reg.get_value()
+            subreg_width = self.sub_regs[0].width
+            sub_regs_value = 0
+            for index, sub_reg in enumerate(self.sub_regs, start=1):
+                if self.reverse_subregs_order:
+                    bit_pos = (index - 1) * subreg_width
+                else:
+                    bit_pos = self.width - index * subreg_width
+                sub_regs_value |= sub_reg.get_value() << (bit_pos)
             if sub_regs_value != self._value:
                 self.set_value(sub_regs_value, raw=True)
 
         return self._value
 
-    def get_int_value(self) -> int:
-        """Get the integer value of register."""
-        return int.from_bytes(self.get_value(), "big")
+    def get_bytes_value(self) -> bytes:
+        """Get the bytes value of register.
+
+        The value indianness is returned by 'reversed' member.
+        :return: Register value in bytes.
+        """
+        endianness = "little" if not self.reverse else "big"
+        return value_to_bytes(
+            self.get_value(),
+            align_to_2n=False,
+            byte_cnt=self.width // 8,
+            endianness=endianness,  # type: ignore[arg-type]
+        )
 
     def get_hex_value(self) -> str:
         """Get the value of register in string hex format."""
-        return format_value(
-            self.get_int_value(), self.width, delimeter="", use_prefix=self.width <= 32
-        )
+        use_prefix = not self.config_as_hexstring
+        return format_value(self.get_value(), self.width, delimiter="", use_prefix=use_prefix)
 
-    def get_reset_value(self) -> bytes:
+    def get_reset_value(self) -> int:
         """Returns reset value of the register.
 
         :return: Reset value of register.
@@ -618,9 +630,14 @@ class RegsRegister:
         :param exclude: Exclude list of bitfield names if needed.
         :return: Returns List of register bitfields.
         """
-        if exclude:
-            return [r for r in self._bitfields if not r.name.startswith(tuple(exclude))]
-        return self._bitfields
+        ret = []
+        for bitf in self._bitfields:
+            if bitf.hidden:
+                continue
+            if exclude and bitf.name.startswith(tuple(exclude)):
+                continue
+            ret.append(bitf)
+        return ret
 
     def get_bitfield_names(self, exclude: List[str] = None) -> List[str]:
         """Returns list of the bitfield names.
@@ -651,32 +668,6 @@ class RegsRegister:
         """
         self._set_value_hooks.append((hook, context))
 
-    def get_bc_bitfield(self, old_name: str) -> Optional[str]:
-        """Try to translate old name of bitfield with the current one.
-
-        :param old_name: Old name to try to find
-        :return: Name of current bitfield if exists
-        """
-        if not self.bc_bitfield:
-            return None
-
-        bitfield_list = self.get_bitfields()
-        for bitfield in bitfield_list:
-            bc_list = self.bc_bitfield(self, bitfield)
-            if old_name in bc_list:
-                return bitfield.name
-        return None
-
-    def enable_backward_compatibility(
-        self,
-        bc_bitfield: Callable[["RegsRegister", RegsBitField], List[str]],
-    ) -> None:
-        """Enables support for basic backward compatibility with older name convention.
-
-        :param bc_bitfield: Function to provide list of back compatibility names of bitfield, defaults to None
-        """
-        self.bc_bitfield = bc_bitfield
-
     def __str__(self) -> str:
         """Override 'ToString()' to print register.
 
@@ -684,18 +675,16 @@ class RegsRegister:
         """
         output = ""
         output += f"Name:   {self.name}\n"
-        # if isinstance(self.alias, str) and self.alias != "":
-        #     output += f"Alias:  {self.alias}\n"
-        # if isinstance(self.type, str) and self.type != "":
-        #     output += f"Type:   {self.type}\n"
         output += f"Offset: 0x{self.offset:04X}\n"
         output += f"Width:  {self.width} bits\n"
         output += f"Access:   {self.access}\n"
         output += f"Description: \n {self.description}\n"
+        if self.otp_index:
+            output += f"OTP Word: \n {self.otp_index}\n"
 
         i = 0
-        for bitfiled in self._bitfields:
-            output += f"Bitfield #{i}: \n" + str(bitfiled)
+        for bitfield in self._bitfields:
+            output += f"Bitfield #{i}: \n" + str(bitfield)
             i += 1
 
         return output
@@ -708,7 +697,6 @@ class Registers:
         """Initialization of Registers class."""
         self._registers: List[RegsRegister] = []
         self.dev_name = device_name
-        self.bc_reg: Optional[Callable[[RegsRegister], List[str]]] = None
 
     def find_reg(self, name: str, include_group_regs: bool = False) -> RegsRegister:
         """Returns the instance of the register by its name.
@@ -721,7 +709,7 @@ class Registers:
         for reg in self._registers:
             if name == reg.name:
                 return reg
-            elif include_group_regs and reg.has_group_registers():
+            if include_group_regs and reg.has_group_registers():
                 for sub_reg in reg.sub_regs:
                     if name == sub_reg.name:
                         return sub_reg
@@ -836,11 +824,83 @@ class Registers:
         for reg in self._registers:
             reg.add_et_subelement(xml_root)
 
-        with open(file_name, "w", encoding="utf-8") as xml_file:
-            no_pretty_data = minidom.parseString(
-                ET.tostring(xml_root, encoding="unicode", short_empty_elements=False)
+        no_pretty_data = minidom.parseString(
+            ET.tostring(xml_root, encoding="unicode", short_empty_elements=False)
+        )
+        write_file(no_pretty_data.toprettyxml(), file_name, encoding="utf-8")
+
+    def image_info(
+        self, size: int = 0, pattern: BinaryPattern = BinaryPattern("zeros")
+    ) -> BinaryImage:
+        """Export Registers into  binary information.
+
+        :param size: Result size of Image, 0 means automatic minimal size.
+        :param pattern: Pattern of gaps, defaults to "zeros"
+        """
+        image = BinaryImage(self.dev_name, size=size, pattern=pattern)
+        for reg in self._registers:
+            image.add_image(
+                BinaryImage(
+                    reg.name,
+                    reg.width // 8,
+                    offset=reg.offset // 8,
+                    description=reg.description,
+                    binary=reg.get_bytes_value(),
+                )
             )
-            xml_file.write(no_pretty_data.toprettyxml())
+
+        return image
+
+    def export(self, size: int = 0, pattern: BinaryPattern = BinaryPattern("zeros")) -> bytes:
+        """Export Registers into binary.
+
+        :param size: Result size of Image, 0 means automatic minimal size.
+        :param pattern: Pattern of gaps, defaults to "zeros"
+        """
+        return self.image_info(size, pattern).export()
+
+    def get_validation_schema(self) -> Dict:
+        """Get the JSON SCHEMA for registers.
+
+        :return: JSON SCHEMA.
+        """
+        properties = {}
+        for reg in self.get_registers():
+            bitfields = reg.get_bitfields()
+            if bitfields:
+                bitfields_schema = {}
+                for bitfield in bitfields:
+                    bitfields_schema[bitfield.name] = {
+                        "type": ["string", "number"],
+                        "title": f"{bitfield.name}",
+                        "description": f"{bitfield.description}",
+                        "template_value": bitfield.get_value(),
+                    }
+                properties[reg.name] = {
+                    "type": "object",
+                    "title": f"{reg.name}",
+                    "description": f"{reg.description}",
+                    "required": ["bitfields"],
+                    "properties": {"bitfields": {"type": "object", "properties": bitfields_schema}},
+                }
+            else:
+                properties[reg.name] = {
+                    "type": "object",
+                    "title": f"{reg.name}",
+                    "description": f"{reg.description}",
+                    "required": ["value"],
+                    "properties": {
+                        "value": {
+                            "type": ["string", "number"],
+                            "title": f"{reg.name}",
+                            "description": f"{reg.description}",
+                            "format": "number",
+                            "template_value": f"{reg.get_hex_value()}",
+                        }
+                    },
+                }
+
+        return {"type": "object", "title": self.dev_name, "properties": properties}
 
     def generate_html(
         self,
@@ -861,7 +921,7 @@ class Registers:
         for reg in self.get_registers(regs_exclude):
             data.append(reg.get_html_data_element(fields_exclude))
 
-        jinja_env = Environment(loader=FileSystemLoader(utils.REGS_DATA_FOLDER))
+        jinja_env = Environment(loader=FileSystemLoader(utils.REGS_DATA_FOLDER), autoescape=True)
         template = jinja_env.get_template("regs_desc_template.html")
         return template.render(heading1=heading1, heading2=heading2, data=data)
 
@@ -875,47 +935,6 @@ class Registers:
         """
         return [item for item in items if not item.attrib["name"].startswith(tuple(names))]
 
-    def enable_backward_compatibility(
-        self, bc_reg: Callable[[RegsRegister], List[str]] = None
-    ) -> None:
-        """Enables support for basic backward compatibility with older name convention.
-
-        :param bc_reg: Function to provide list of back compatibility names of register, defaults to None
-        """
-        self.bc_reg = bc_reg
-
-    def enable_backward_compatibility_enums(
-        self, bc_enum: Callable[[RegsRegister, RegsBitField, RegsEnum], List[str]]
-    ) -> None:
-        """Enables support for basic backward compatibility with older name convention.
-
-        :param bc_enum: Function to provide list of back compatibility names of all enums, defaults to None
-        """
-        for reg in self.get_registers():
-            for bitfield in reg.get_bitfields():
-                bitfield.enable_backward_compatibility(bc_enum)
-
-    def get_bc_reg(self, old_name: str, regs: List[str] = None) -> Optional[str]:
-        """Try to translate old name of register with the current one.
-
-        :param old_name: Old name to try to find
-        :param regs: Optional set of regs name if the full list is not used.
-        :return: Name of current register
-        """
-        if not self.bc_reg:
-            return None
-
-        reg_list = (
-            [r for r in self.get_registers(include_group_regs=True) if r.name in regs]
-            if regs
-            else self.get_registers(include_group_regs=True)
-        )
-        for reg in reg_list:
-            bc_list = self.bc_reg(reg)
-            if old_name in bc_list:
-                return reg.name
-        return None
-
     # pylint: disable=dangerous-default-value
     def load_registers_from_xml(
         self, xml: str, filter_reg: List[str] = None, grouped_regs: List[dict] = None
@@ -925,7 +944,7 @@ class Registers:
         :param xml: Input XML data in string format.
         :param filter_reg: List of register names that should be filtered out.
         :param grouped_regs: List of register prefixes names to be grouped int one.
-        :raises SPSDKRegsError: XML parse problem occuress.
+        :raises SPSDKRegsError: XML parse problem occurs.
         """
 
         def is_reg_in_group(reg: str) -> Union[dict, None]:
@@ -939,7 +958,7 @@ class Registers:
         try:
             xml_elements = ET.parse(xml)
         except ET.ParseError as exc:
-            raise SPSDKRegsError(f"Cannot Parse XML data: {str(exc)}")
+            raise SPSDKRegsError(f"Cannot Parse XML data: {str(exc)}") from exc
         xml_registers = xml_elements.findall("register")
         xml_registers = self._filter_by_names(xml_registers, filter_reg or [])
         # Load all registers into the class
@@ -958,7 +977,10 @@ class Registers:
                         ),
                         reverse=value_to_bool(group.get("reverse", False)),
                         access=group.get("access", None),
+                        config_as_hexstring=group.get("config_as_hexstring", False),
+                        reverse_subregs_order=group.get("reverse_subregs_order", False),
                     )
+
                     self.add_register(group_reg)
                 group_reg.add_group_reg(RegsRegister.from_xml_element(xml_reg))
             else:
@@ -975,43 +997,20 @@ class Registers:
         :param yml_data: The YAML commented data with register values.
         :param exclude_regs: List of excluded registers
         :param exclude_fields: Dictionary with lists of excluded bitfields
-        :raises SPSDKRegsErrorRegisterNotFound: Register in configuration has not been found.
-        :raises SPSDKRegsErrorBitfieldNotFound: Bitfield in configuration has not been found.
         """
-        regs = self.get_reg_names(exclude_regs, include_group_regs=True)
         for reg_name in yml_data.keys():
+            if exclude_regs and reg_name in exclude_regs:
+                continue
             reg_dict = yml_data[reg_name]
-            if reg_name not in regs:
-                # check also the backward compatibility of this register in configuration
-                found_name = self.get_bc_reg(reg_name, regs)
-                if not found_name:
-                    if exclude_regs and reg_name.startswith(tuple(exclude_regs)):
-                        continue
-                    raise SPSDKRegsErrorRegisterNotFound(
-                        f"The {reg_name} register has not found in current configuration."
-                    )
-
-                logger.warning(
-                    f"The {reg_name} in configuration has been replaced by {found_name}!"
-                )
-                reg_name = found_name
-            # The loaded register is our
             register = self.find_reg(reg_name, include_group_regs=True)
             if "value" in reg_dict.keys():
-                val = value_to_bytes(reg_dict["value"])
+                raw_val = reg_dict["value"]
+                val = int(raw_val, 16) if register.config_as_hexstring else value_to_int(raw_val)
                 register.set_value(val, True)
             elif "bitfields" in reg_dict.keys():
                 for bitfield_name in reg_dict["bitfields"]:
-                    try:
-                        bitfield_val = reg_dict["bitfields"][bitfield_name]
-                        bitfield = register.find_bitfield(bitfield_name)
-                    except SPSDKRegsErrorBitfieldNotFound as exc:
-                        # check also the backward compatibility of this bitfield in configuration
-                        found_name = register.get_bc_bitfield(bitfield_name)
-                        if not found_name:
-                            raise SPSDKRegsErrorBitfieldNotFound(exc.description) from exc
-                        bitfield_name = found_name
-                        bitfield = register.find_bitfield(bitfield_name)
+                    bitfield_val = reg_dict["bitfields"][bitfield_name]
+                    bitfield = register.find_bitfield(bitfield_name)
                     if (
                         exclude_fields
                         and reg_name in exclude_fields.keys()
@@ -1042,7 +1041,6 @@ class Registers:
                 "&#10;", f"\n{' '*indent}# "
             )
         if bitfield.has_enums():
-            # description += new_line("Possible Enumeration values 'NAME, (VALUE): Description'")
             for enum in bitfield.get_enums():
                 descr = enum.description if enum.description != "." else enum.name
                 enum_description = descr.replace("&#10;", f"\n{' '*indent}# ")
@@ -1062,7 +1060,7 @@ class Registers:
         :return: YAML commented map of registers value.
         """
         data = CM()
-        ix = 0
+        index = 0
         for reg in self.get_registers():
             if white_list and reg.name not in white_list.keys():
                 continue
@@ -1072,12 +1070,12 @@ class Registers:
             descr = reg.description if reg.description != "." else reg.name
             descr = descr.replace("&#10;", "\n# ")
             data.insert(
-                ix,
+                index,
                 reg.name,
                 reg_yml,
                 comment=descr,
             )
-            ix += 1
+            index += 1
             bitfields = reg.get_bitfields()
             if len(bitfields) > 0 and white_list and white_list[reg.name]:
                 btf_yml = CM()
